from pathlib import Path
from itertools import chain
import logging


_logger = logging.getLogger(__name__)


def filter_installable(module):
    manifest = get_manifest(module)
    return manifest.get('installable', False)


def filter_noninstallable(module):
    return not filter_installable(module)


def get_filter(filter_names):
    filters = []

    if 'installable' in filter_names:
        filters.append(filter_installable)

    if 'non_installable' in filter_names:
        filters.append(filter_noninstallable)

    if not filter_names:
        return lambda module: True

    def filter_module(module):
        return all([check(Path.cwd() / module) for check in filters])

    return filter_module


def find_modules(path, only_name=False, filters=None):
    modules = set()

    path = Path.cwd() / path

    erp_manifest = '__openerp__.py'
    odoo_manifest = '__manifest__.py'

    manifest_globs = chain(
        path.glob('**/{}'.format(erp_manifest)),
        path.glob('**/{}'.format(odoo_manifest)),
    )

    check_module = get_filter(filters)

    for path in manifest_globs:
        rel_path = path.parent.relative_to(Path.cwd())

        if not check_module(rel_path):
            continue

        if only_name:
            modules.add(rel_path.name)
        else:
            modules.add(str(rel_path))

    return modules


def find_modules_paths(paths, only_name=False, filters=None):
    modules = set()

    for path in paths:
        modules = modules.union(
            find_modules(path, only_name=only_name, filters=filters)
        )

    return modules


def get_manifest(module_path, render_description=False):
    odoo_manifest = module_path / '__manifest__.py'
    erp_manifest = module_path / '__openerp__.py'

    manifest = None
    if odoo_manifest.exists():
        manifest = odoo_manifest
    if not manifest and erp_manifest.exists():
        manifest = erp_manifest

    try:
        with manifest.open('r') as fin:
            code = compile(fin.read().replace('\ufeff', ''), '__manifest__.py', 'eval')
            data = eval(code, {}, {})
            data['module_name'] = data['name']
            data['name'] = module_path.name
    except SyntaxError:
        _logger.error("Cannot parse manifest: {}".format(manifest), exc_info=True)
        data = {}
    except AttributeError:
        data = {}

    if render_description:
        data['description_html'] = render_description(
            module_path, data.get('description', '')
        )

    return data
